import functools
import logging
import pickle
import time
from collections import namedtuple
from datetime import datetime
from inspect import getfullargspec
from typing import Dict, List, Tuple

import lxml.etree
import requests

from .exceptions import LoginFailedError, PasswordError
from .utils import extract_string

WorkInfo = namedtuple(
    "WorkInfo", ["workName", "startTime", "endTime", "workStatus"])
CourseInfo = namedtuple(
    "CourseInfo", ["pageUrl", "courseName", "teacherName", "courseSeq"])


def cache(expire_time):
    """缓存函数返回值。在expire_time时间内重复调用某个函数（且str(参数)相同）会使用上次的返回值。
    @expire_time 缓存过期时间，单位：秒。

    例子：\n
    @cache(60)\n
    def get_val(x, y, z):
        # something
        time.sleep(1)
        return [x, y, z]
    如果在60s内，重复调用该函数，并且x,y,z值相同的情况下，会直接使用上次运行的返回值。

    在上述例子中，执行10次get_val(1, 2, 3)只消耗约1秒时间。
    """
    def decorator(func):
        @functools.wraps(func)
        def wrap(self, *vargs, **kwargs):
            if "disable_cache" in kwargs.keys():
                disable_cache = kwargs["disable_cache"]
            else:
                disable_cache = False
            key = func.__name__
            for x in vargs:
                key += str(x)
            for x in kwargs.keys():
                key += str(x)
                key += str(kwargs[x])
            cache = self._read_cache(key, expire_time=expire_time)
            if (not disable_cache) and (cache is not None):
                self._logger.debug(f"{func.__name__} 使用缓存值。key: {key}")
                return cache
            else:
                self._logger.debug(f"call {func.__name__}...")
                r = func(self, *vargs, **kwargs)
                self._write_cache(key, r)
                self.last_update_time = time.time()
                return r
        wrap.origin = func
        return wrap
    return decorator


class ChaoxingUser:
    HTTP_HEADERS = {
        # dummy user-agent
        'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13) AppleWebKit/603.1.13 (KHTML, like Gecko) Version/10.1 Safari/603.1.13',
        "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8,zh-TW;q=0.7"
    }
    CACHE_EXPIRE_TIME = 600

    def __init__(self, userName, password):
        self.userName = userName
        self.password = password
        self.session = requests.Session()
        self.session.headers.update(self.HTTP_HEADERS)
        self._logger = logging.getLogger(__name__)
        self._cache_table: Dict[str, Tuple[List, float]] = dict()
        # 如果该对象是通过load_from产生的，load_file 为其来源文件，否则 load_from 为空。
        self.load_file = ""
        self.last_update_time = 0

    def http_request(self, url, method, params=None, data=None, referer=None, auto_retry=3) -> requests.models.Response:
        session = self.session
        if referer:
            session.headers.update({
                "Referer": referer
            })
        request = getattr(session, method.lower())
        while True:
            try:
                r = request(url, params=params, data=data)
                return r
            except requests.exceptions.RequestException as e:
                self._logger.error(f"请求时发生错误：{e}")
                if isinstance(e, ValueError):
                    raise
                if auto_retry > 0:
                    time.sleep(1)
                    auto_retry -= 1
                    self._logger.error(f"自动重试: {auto_retry}")
                else:
                    raise

    def http_get(self, url, **kargs):
        return self.http_request(url, "get", **kargs)

    def http_post(self, url, **kargs):
        return self.http_request(url, "post", **kargs)

    def _read_cache(self, key, expire_time=None):
        if key in self._cache_table.keys():
            cached_time = self._cache_table[key][1]
            if expire_time is None:
                expire_time = self.CACHE_EXPIRE_TIME
            if (time.time() - cached_time) < expire_time:
                return self._cache_table[key][0].copy()
        return None

    def _write_cache(self, key, val: list):
        self._cache_table[key] = (val.copy(), time.time())

    @property
    def is_login(self):
        r = self.http_get("http://www.elearning.shu.edu.cn/topjs?index=1")
        if "afterLogin" in r.text:
            return True
        else:
            return False

    def login(self):
        # step 1
        r = self.http_get("http://shu.fysso.chaoxing.com/sso/shu",
                          referer="http://www.elearning.shu.edu.cn/portal")
        if r.url.startswith("http://www.elearning.shu.edu.cn/sso/logind"):
            self._logger.debug("use oauth to login")
        elif r.url == "https://oauth.shu.edu.cn/login":
            # step 2
            self._logger.debug("use username and password to login")
            form = {
                "username": self.userName,
                "password": self.password,
                "login_submit": "登录/Login"
            }
            r = self.http_post("https://oauth.shu.edu.cn/login", data=form)
            if "认证失败" in r.text:
                raise PasswordError
            if "连续出错次数太多" in r.text:
                raise LoginFailedError(2, "连续出错次数太多")
            if not r.url.startswith("http://www.elearning.shu.edu.cn/sso/logind"):
                raise LoginFailedError(2, f"unexpected url(2): {r.url}")
        else:
            error = f"login failed. unexpected url(1): {r.url}"
            self._logger.critical(error)
            raise LoginFailedError(1, error)

        # step 3
        # r.url start with http://www.elearning.shu.edu.cn/sso/logind
        html = lxml.etree.HTML(r.text)
        form = html.xpath("//form[@id='userLogin']")[0]
        inputs = form.xpath("input[@type='hidden']")
        request_url = form.attrib["action"]
        data = dict()
        for i in inputs:
            name = i.attrib["name"]
            value = i.attrib["value"]
            data[name] = value
            if name == "fid":
                fid = value
        r = self.http_post(request_url, data=data)

        if r.url != "http://www.elearning.shu.edu.cn/portal":
            raise LoginFailedError(3, f"unexpected url(3): {r.url}")

        # step 4
        params = {"fid": fid}
        self.http_get(
            "http://www.elearning.shu.edu.cn/setcookie.jsp", params=params)

    @cache(600)
    def get_term_id_list(self, disable_cache=False) -> List[Tuple[int, str]]:
        """获取学期id
        @return [(20193, "2019-2020学年春季学期"), (20192, "2019-2020学年秋季学期"), ...]
        """
        # step 1 获取请求url
        r = self.http_get("http://i.mooc.elearning.shu.edu.cn/space/index.shtml",
                          referer="http://www.elearning.shu.edu.cn/portal")
        url = extract_string(
            r.text, "http://www.elearning.shu.edu.cn/courselist/study?s=")
        self._logger.info(f"get_term_id_list request url: {url}")
        # step 2
        result = list()
        r = self.http_get(url)
        html = lxml.etree.HTML(r.text)
        term_list_li = html.xpath(
            "//ul[@class='zse_ul']/li[@class='zse_li']/a")
        for x in term_list_li:
            year = x.xpath("@data_year")[0]
            term = x.xpath("@data_term")[0]
            comment = x.text.strip()
            try:
                term_id = int(year + term)
            except ValueError as e:
                self._logger.error(f"term_id转换错误：{e}, 略过：{comment}")
                term_id = -2
            result.append((term_id, comment))
        result.sort(key=lambda x: x[0], reverse=True)
        self._logger.info(f"term_id: {result}")
        return result

    @cache(600)
    def get_course_list(self, term_id: int = -1, disable_cache=False) -> List[CourseInfo]:
        """获取课程列表
        @params term_id 学期ID，-1表示当前学期, 0表示所有课程。20193表示2019-2020春季学期。

        @return [CourseInfo(pageUrl='', courseName='', teacherName='', courseSeq=''), ...]
        """
        # step 1 获取请求url
        r = self.http_get("http://i.mooc.elearning.shu.edu.cn/space/index.shtml",
                          referer="http://www.elearning.shu.edu.cn/portal")
        url = extract_string(
            r.text, "http://www.elearning.shu.edu.cn/courselist/study?s=")
        self._logger.info(f"get_course_list request url: {url}")

        # step 2
        term_id_list = self.get_term_id_list()
        request_data = {
            "year": "0",
            "term": "0",
            "showContent": "000"
        }
        if term_id == -1:
            term_id = term_id_list[0][0]
        if term_id // 10000 == 2:
            request_data["year"] = str(term_id // 10)
            request_data["term"] = str(term_id % 10)
        elif term_id != 0:
            self._logger.error(f"invalid term_id: {term_id}")
            raise ValueError(f"invalid term_id: {term_id}")
        self._logger.debug(f"get_course_list term_id: {term_id}")

        r = self.http_get(url, params=request_data)
        html = lxml.etree.HTML(r.text)
        courses = html.xpath("//li[contains(@class, 'zmy_item')]")
        course_list = list()
        for c in courses:
            course_list.append(CourseInfo(
                pageUrl=c.xpath("a/@href")[0],
                courseName=c.xpath(
                    "dl/dt[@name='courseNameHtml']")[0].text.strip(),
                teacherName=c.xpath(
                    "dl/dd[@name='userNameHtml']")[0].text.strip(),
                courseSeq=c.xpath("dl/dt/span/text()")[0].strip()[1:-1]
            ))
        return course_list

    @cache(600)
    def get_work_list(self, page_url, disable_cache=False) -> List[WorkInfo]:
        """获取作业列表
        """
        result = list()
        r = self.http_get(page_url)
        # step 1 获取url
        request_path = extract_string(r.text, "/work/getAllWork?")
        request_url = "http://mooc1.elearning.shu.edu.cn" + request_path
        self._logger.info(f"get_work_list request_url: {request_url}")
        # step 2
        r = self.http_get(request_url)
        html = lxml.etree.HTML(r.text)
        works = html.xpath("//div[@class='ulDiv']/ul/li")
        self._logger.info(f"get_work_list len(works) = {len(works)}")
        for x in works:
            workName = x.xpath("div[@class='titTxt']/p/a/@title")[0]
            t = x.xpath("div[@class='titTxt']/span[@class='pt5']/text()")
            t = [i.strip() for i in t]
            time_format = r"%Y-%m-%d %H:%M"
            startTime = datetime.strptime(t[0], time_format) if t[0] else None
            endTime = datetime.strptime(t[1], time_format) if t[1] else None
            workStatus = x.xpath(
                "div[@class='titTxt']/span/strong")[0].text.strip()
            result.append(WorkInfo(
                workName=workName,
                startTime=startTime,
                endTime=endTime,
                workStatus=workStatus
            ))
        return result

    @cache(600)
    def get_unfinish_work_list(self, term_id=-1, disable_cache=False):
        """获取未完成作业。即状态为：待做
        @return [(CourseInfo, [WorkInfo, ...]), ...]
        """
        course_list = self.get_course_list(
            term_id=term_id, disable_cache=disable_cache)
        result = list()
        for course in course_list:
            work_list = self.get_work_list(
                course.pageUrl, disable_cache=disable_cache)
            unfinish_works = [x for x in work_list if x.workStatus == "待做"]
            if unfinish_works:
                result.append((course, unfinish_works))
        return result

    def dump_to(self, file_path=None):
        if file_path is None:
            file_path = self.load_file
        if file_path is None:
            raise ValueError("need param file_path")
        self._logger.debug(f"dump object to {file_path}")
        with open(file_path, "wb") as f:
            # 你问我为啥要删掉 _logger?
            # 我也不知道。反正不删的话在 python3.6 中会报错 TypeError: can't pickle _thread.RLock objects。
            # 大概是因为 pickle 不能序列化 Logger 里的 _thread.Rlock 对象吧。
            del self._logger
            pickle.dump(self, f)

    @staticmethod
    def load_from(file_path):
        with open(file_path, "rb") as f:
            obj = pickle.load(f)
            obj.load_file = file_path
            if not hasattr(obj, "_logger"):
                obj._logger = logging.getLogger(__name__)
            return obj



import setuptools
import ohmyddl
import pathlib


with open("README.md", "r", encoding="utf-8") as file:
    long_description = file.read()


webroot = pathlib.Path("ohmyddl/webroot")
webroot_data = [str(x)[8:] for x in webroot.glob("**/*")]


setuptools.setup(
    name="ohmyddl",
    version=ohmyddl.__version__,
    author="hwenwur",
    author_email="pypi@qiren.org",
    description="A tool to collect deadline in SHU.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/hwenwur/oh-my-ddl",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        "lxml",
        "requests",
    ],
    entry_points={
        "console_scripts":[
            "ohmyddl-cli=ohmyddl.__main__:main",
            "ohmyddl=ohmyddl.__main__:web"
        ]
    },
    include_package_data=True,
    package_data = {
        "ohmyddl": webroot_data
    }
)



import functools
import logging
import random
import string
from pathlib import Path

from .bottle import hook, post, request, response, run, route, static_file

from . import DATA_DIR, exceptions
from .models import ChaoxingUser
from .utils import geanerate_sid, check_sid_format


_ret_code = {
    0: "OK",
    1: "invalid token",
    2: "username or password error",
    3: "invalid request message",
    -1: "unknown error"
}
logger = logging.getLogger(__name__)
web_root = (Path(__file__).parent / Path("webroot")).resolve()


def setup_logging(level=logging.ERROR):
    root = logging.getLogger()
    root.addHandler(logging.StreamHandler())
    root.setLevel(level)


def make_response(ret_code, extra_message=None, body=None):
    message = _ret_code[ret_code]
    if extra_message:
        message += "," + extra_message
    result = {
        "ret": ret_code,
        "message": message
    }
    if body:
        result.update(body)
    return result


def login_required(func):
    """login_required 装饰器，对需要登录的路由使用可在 request 对象中增加属性：user,
    并保证 user 可用。
    """
    @functools.wraps(func)
    def wrap(*vargs, **kwargs):
        sid = request.get_cookie("sid")
        ret = 1
        message = ""
        logger.debug("start check sid...")
        if check_sid_format(sid):
            file = DATA_DIR / Path(sid)
            if file.exists():
                request.sid = sid
                user: ChaoxingUser = ChaoxingUser.load_from(file)
                if not user.is_login:
                    logger.debug("login...")
                    try:
                        user.login()
                        request.user = user
                        return func(*vargs, **kwargs)
                    except exceptions.PasswordError:
                        ret = 2
                    except exceptions.LoginFailedError as e:
                        ret = -1
                        message = str(e)
                else:
                    request.user = user
                    return func(*vargs, **kwargs)
            else:
                logger.debug("token not exists")
        else:
            logger.debug("token format error")
        
        logger.debug("check sid end")
        return make_response(ret, message)
    return wrap


def json_required(func):
    """josn_required 装饰器：如果 content-type 不是 application/json, 直接返回失败。
    如果请求体格式实际不是 json, 也会返回失败。
    """
    @functools.wraps(func)
    def wrap(*vargs, **kwargs):
        ret = 3
        message = ""
        try:
            tmp = request.json
            if tmp is None:
                logger.debug("request message is None")
            return func(*vargs, **kwargs)
        except ValueError as e:
            message = str(e)
        return make_response(ret, message)
    return wrap


@hook("after_request")
def after_request():
    if hasattr(request, "user"):
        user = request.user
        user.dump_to()
        logger.debug(f"save user object to local drive success.")


@post("/api/login")
@json_required
def login():
    ret = -1
    message = ""
    try:
        payload = request.json
        username = payload["username"] #pylint: disable=unsubscriptable-object
        password = payload["password"] #pylint: disable=unsubscriptable-object
        message = ""
    except KeyError as e:
        username = password = None
        message = f"错误：{e.args}"
    print(f"login: {username} - {password}")
    if username and password and len(username) == 8:
        user = ChaoxingUser(username, password)
        try:
            user.login()
            # 随机序列
            randstr = geanerate_sid()
            file = DATA_DIR / Path(randstr)
            user.dump_to(file)
            response.set_cookie("sid", randstr, max_age=100 * 24 * 60 * 60)
            return make_response(0)
        except exceptions.PasswordError:
            ret = 2
        except exceptions.LoginFailedError as e:
            message = str(e)
    else:
        ret = 2
    return make_response(ret, message)


@post("/api/check_sid")
@json_required
def check_sid():
    sid = request.get_cookie("sid")
    ret = 0
    result = { 
        "ret": ret, 
        "message": _ret_code[ret],
        "sid_available": True
    }
    if check_sid_format(sid):
        file = DATA_DIR / Path(sid)
        if file.exists():
            return result
        else:
            logger.debug("sid not exists")
    else:
        logger.debug("invalid sid format.")
    result["sid_available"] = False
    return result


@post("/api/get_unfinish_works")
@login_required
@json_required
def get_unfinish_works():
    disable_cache = False
    if request.json and ("disable_cache" in request.json.keys()): #pylint: disable=no-member
        disable_cache = request.json["disable_cache"] #pylint: disable=unsubscriptable-object
    user = request.user
    course_list = user.get_unfinish_work_list(disable_cache=disable_cache)
    result = {"ret": 0}
    data = []
    for course, works in course_list:
        for w in works:
            deadline = w.endTime
            if deadline is None:
                deadline = -1
            else:
                deadline = deadline.timestamp()
            data.append({
                "courseName": course.courseName,
                "workName": w.workName,
                "deadline": deadline
            })
    data.sort(key=lambda x: x["deadline"])
    result["data"] = data
    result["update_at"] = user.last_update_time
    return result


@post("/api/logout")
def logout():
    if hasattr(request, "sid"):
        sid = request.sid
        file = DATA_DIR / Path(sid)
        file.unlink()
        del request.sid
        del request.user
        logger.debug(f"file {file} deleted.")
    response.set_cookie("sid", "")
    return {
        "ret": 0,
        "message": _ret_code[0]
    }


# static file
@route("/")
def home():
    logger.debug(f"webroot: {web_root}")
    return static_file("index.html", root=web_root)


@route("/<filename>")
def resource_file(filename):
    return static_file(filename, root=web_root)


@route("/static/<filepath:path>")
def resource_file2(filepath):
    return static_file(filepath, root=(web_root / Path("static")))


def main(host="localhost", port=5986):
    setup_logging(level=logging.DEBUG)
    run(host=host, port=port)


if __name__ == "__main__":
    main()



import random
import string
import sys
import threading
from io import StringIO
from pathlib import Path
from typing import List

import requests

from . import DATA_DIR

# ---------------------------------------------


alias_table = {
    "毛泽东思想和中国特色社会主义理论体系概论*": "毛概",
    "马克思主义基本原理概论*": "马原",
    "中国近现代史纲要*": "中近纲",
    "数据结构与算法": "数据结构",
    "电路*": "电路",
    "数字电子技术": "数电",
    "模拟电子技术*": "模电"
}


def get_course_alias(course_name: str):
    keys = alias_table.keys()
    if course_name in keys:
        return alias_table[course_name]
    for k in keys:
        if k.endswith("*"):
            if course_name.startswith(k[:-1]):
                return alias_table[k]
        if k.startswith("*"):
            if course_name.endswith(k[1:]):
                return alias_table[k]
    return course_name


# ----------------------------------------------


def extract_string(source, start):
    """从source中提取第一个以start开头的字符串（单引号或双引号包围的）。
    """
    s = source.index(start)
    quote = source[s - 1]
    if quote == "'" or quote == '"':
        temp = source[s:]
        e = temp.index(quote)
        return temp[:e]
    else:
        raise ValueError


# -----------------------------------------------


def display_length(content):
    """中文单字长度计为2
    """
    content = str(content)
    length = len(content)
    for x in content:
        if 0x4e00 <= ord(x) <= 0x9fa5:
            length += 1
    return length


def print_width(stream: StringIO, content, width: int, align=0):
    content = str(content)
    content_len = display_length(content)
    if content_len <= width:
        a = (width - content_len) // 2
        if align == 0:
            # 居中
            stream.write(" " * a)
            stream.write(content)
            stream.write(" " * a)
        elif align == -1:
            # 左对齐
            stream.write(content)
            stream.write(" " * a * 2)
        elif align == 1:
            # 右对齐
            stream.write(" " * a * 2)
            stream.write(content)
        real_width = a * 2 + display_length(content)
        padding = width - real_width
        if padding > 0:
            stream.write(" " * padding)
    else:
        print(f"WARN content={content},length={content_len},width={width}")
        stream.write(content)


def table(data: List[List[str]], header: List[str]):
    col_max_len = [display_length(x) for x in header]
    for row in data:
        for i, col in enumerate(row):
            if display_length(col) > col_max_len[i]:
                col_max_len[i] = display_length(col)
    col_max_len = [2 + x for x in col_max_len]
    out = StringIO()
    # line 1
    out.write("+")
    for x in col_max_len:
        out.write("-" * x)
        out.write("+")
    out.write("\n")
    # line 2
    out.write("|")
    for i, h in enumerate(header):
        print_width(out, h, col_max_len[i])
        out.write("|")
    out.write("\n")
    # lin 3
    out.write("|")
    for x in col_max_len:
        out.write("-" * x)
        out.write("+")
    out.seek(out.tell() - 1)
    out.write("|\n")
    # line [4,len - 1)
    for row in data:
        out.write("|")
        for i, col in enumerate(row):
            print_width(out, col, col_max_len[i], align=-1)
            out.write("|")
        out.write("\n")
    # line len - 1, (the last line.)
    out.write("+")
    for x in col_max_len:
        out.write("-" * x)
        out.write("+")
    out.write("\n")

    return out.getvalue()

# -----------------------------------------------

def fetch_term_desc(term_id_list, term_id):
    if term_id == -1:
        return term_id_list[0][1]
    elif term_id:
        return "不限"
    for term in term_id_list:
        if term[0] == term_id:
            return term[1]
    return "未知"

# -----------------------------------------------

def geanerate_sid():
    """生成随机32字节小写字母。例如：ruzztvxjrptwryccrrgdbyuzkoozfgua
    """
    return ''.join([random.choice(string.ascii_lowercase) for x in range(32)])


def check_sid_format(sid):
    if sid is not None and isinstance(sid, str) and len(sid) == 32 and sid.isalpha():
        return True
    return False


# -----------------------------------------------

def get_user_id():
    file = DATA_DIR / Path("userid")
    if file.exists() and file.is_file():
        with file.open("r", encoding="utf-8") as fd:
            uid = fd.read()
            if len(uid) == 32:
                return uid
    elif not file.exists():
        # 这里使用和 sid 相同的格式
        uid = geanerate_sid()
        with file.open("w", encoding="utf-8") as fd:
            fd.write(uid)
        return uid
    else:
        # 其他情况，userid是一个文件夹等。当作用户不想被统计。
        pass
    return None


def check_available(blocking=False):
    """判断本程序是否可用。只有得到服务器明确回应后才关闭。
    """
    def print_message():
        print("=============================")
        print("      本程序不再开放使用        ")
        print("=============================")

    file = DATA_DIR / Path("disable")
    if file.exists():
        print_message()
        sys.exit(0)

    if not blocking:
        t = threading.Thread(target=check_available, args=(True, ))
        t.start()
        return
    url = "http://api.qiren.org/ohmyddl/can_i_use"
    # wumingshi - 无名氏
    uid = get_user_id() or "wumingshi"
    params = { "uid":  uid}
    try:
        r = requests.get(url, params=params)
        if r.status_code == 200 and r.text == "no":
            with file.open("a"):
                pass
            print_message()
            sys.exit(0)
    except:
        pass



import argparse
import logging
import sys
from datetime import datetime
from getpass import getpass
from pathlib import Path

from . import DATA_DIR
from .exceptions import PasswordError
from .models import ChaoxingUser, CourseInfo, WorkInfo
from .utils import fetch_term_desc, get_course_alias, table, check_available

DATA_FILE = DATA_DIR / ".user_data"
logger = logging.getLogger(__name__)
check_available()


def setup_logging(level=logging.ERROR):
    root = logging.getLogger()
    root.addHandler(logging.StreamHandler())
    root.setLevel(level)


def solve_account(relogin=False) -> ChaoxingUser:
    if (not relogin) and DATA_FILE.exists():
        logging.debug(f"read config from {DATA_FILE}")
        user = ChaoxingUser.load_from(str(DATA_FILE))
    else:
        username = input("学号：")
        password = getpass("密码：")
        if len(username) != 8:
            print("学号错误")
            exit(1)
        user = ChaoxingUser(username, password)
        try:
            user.login()
        except PasswordError:
            print("学号或密码错误")
            exit(0)
    try:
        if not user.is_login:
            user.login()
        print("登录成功")
    except Exception as e:
        print(f"登录失败：{e}")
        exit(1)
    return user


def save_user_data(user):
    parent = DATA_FILE.parent
    parent.mkdir(exist_ok=True)
    user.dump_to(str(DATA_FILE))


def main():
    prog = "ohmyddl.exe" if sys.platform == "win32" else "ohmyddl"
    parser = argparse.ArgumentParser(description="超星学习通作业汇总。", prog=prog)
    parser.add_argument("-c", help="不使用已保存学号", action="store_true")
    parser.add_argument(
        "-f", help="强制刷新（若10分钟内查询过，会优先使用缓存的数据）", action="store_true")
    parser.add_argument(
        "-t", help="学期ID，默认使用当前学期。格式：20192表示2019冬季学期。", type=int)
    parser.add_argument("-v", help="输出调试信息", action="store_true")
    args = parser.parse_args()
    if args.v:
        setup_logging(logging.DEBUG)
    else:
        setup_logging()

    user = solve_account(relogin=args.c)
    term_id = args.t if args.t else -1
    disable_cache = args.f
    logger.info(f"disable_cache: {disable_cache}")
    term_id_list = user.get_term_id_list(disable_cache=disable_cache)
    course_list = user.get_course_list(
        term_id=term_id, disable_cache=disable_cache)
    work_list = dict()
    for course in course_list:
        works = user.get_work_list(course.pageUrl, disable_cache=disable_cache)
        work_list[course.courseName] = works

    no_work_course = list()
    tab_content = list()
    for course_name in work_list.keys():
        course_alias = get_course_alias(course_name)
        works = work_list[course_name]
        unfinish_works = [x for x in works if x.workStatus == "待做"]
        if len(unfinish_works) == 0:
            no_work_course.append(course_alias)
            continue
        for work in unfinish_works:
            tab_content.append([
                course_alias + "-" + work.workName,
                work.endTime if work.endTime is not None else "未知"
            ])
    tab_content.sort(key=lambda x: x[1].timestamp() if isinstance(x[1], datetime) else 0x7fffffff)
    print(f"当前学号：{user.userName}")
    print(f"当前学期：{fetch_term_desc(term_id_list, term_id)}")
    print(table(tab_content, ["名称", "截止时间"]))
    print()
    if len(no_work_course) != 0:
        print(f"{'、'.join(no_work_course)}没有未完成作业。")
    save_user_data(user)


def web():
    from . import server
    import webbrowser
    webbrowser.open("http://localhost:5986/")
    server.main()


if __name__ == "__main__":
    main()
